@@ -1,4 +1,4 @@
-getTree <- function(rfobj, k=1) {
+getTree <- function(rfobj, k=1, labelVar=FALSE) {
   if (is.null(rfobj$forest)) {
     stop("No forest component in ", deparse(substitute(rfobj)))
   }
@@ -20,8 +20,22 @@ getTree <- function(rfobj, k=1) {
                     rfobj$forest$nodestatus[,k],
                     rfobj$forest$nodepred[,k])[1:rfobj$forest$ndbigtree[k],]
   }      
-  colnames(tree) <- c("left daughter", "right daughter", "split var",
-                      "split point", "status", "prediction")
+
+  dimnames(tree) <- list(1:nrow(tree), c("left daughter", "right daughter",
+                                         "split var", "split point",
+                                         "status", "prediction"))
+
+  if (labelVar) {
+      tree <- as.data.frame(tree)
+      v <- tree[[3]]
+      v[v == 0] <- NA
+      tree[[3]] <- factor(rownames(rfobj$importance)[v])
+      if (rfobj$type == "classification") {
+          v <- tree[[6]]
+          v[! v %in% 1:nlevels(rfobj$y)] <- NA
+          tree[[6]] <- levels(rfobj$y)[v]
+      }
+  }
   tree
 }
 
